@@ -60,6 +60,7 @@ class SwiftFTPShell:
         self.swiftconn = swiftconn
         self.swiftfilesystem = SwiftFileSystem(self.swiftconn)
         self.log_command('login')
+        log.msg(metric='num_clients')
 
     def log_command(self, command, *args):
         arg_list = ', '.join(str(arg) for arg in args)
@@ -69,6 +70,10 @@ class SwiftFTPShell:
 
     def logout(self):
         self.log_command('logout')
+        log.msg(metric='num_clients', count=-1)
+        if self.swiftconn.pool:
+            self.swiftconn.pool.closeCachedConnections()
+        del self.swiftconn
 
     def _fullpath(self, path_parts):
         return '/'.join(path_parts)
@@ -100,8 +105,10 @@ class SwiftFTPShell:
         fullpath = self._fullpath(path)
 
         def errback(failure):
-            failure.trap(NotFound)
-        d = self.swiftfilesystem.removeFile(fullpath)
+            failure.trap(NotFound, NotImplementedError)
+            if failure.check(NotImplementedError):
+                return defer.fail(IsADirectoryError(fullpath))
+        d = defer.maybeDeferred(self.swiftfilesystem.removeFile, fullpath)
         d.addErrback(errback)
         return d
 
@@ -248,7 +255,7 @@ class SwiftReadFile(Protocol):
 
     def dataReceived(self, data):
         self.consumer.write(data)
-        log.msg(metric='egress_bytes', count=len(data))
+        log.msg(metric='transfer.egress_bytes', count=len(data))
 
     def connectionLost(self, reason):
         from twisted.web._newclient import ResponseDone
