@@ -3,7 +3,7 @@ This file defines what is required for swftp-sftp to work with twistd.
 
 See COPYING for license information.
 """
-from swftp.utils import VERSION
+from swftp import VERSION
 
 from twisted.application import internet, service
 from twisted.python import usage, log
@@ -16,6 +16,27 @@ import sys
 import time
 
 
+CONFIG_DEFAULTS = {
+    'auth_url': 'http://127.0.0.1:8080/auth/v1.0',
+    'host': '0.0.0.0',
+    'port': '5022',
+    'priv_key': '/etc/swftp/id_rsa',
+    'pub_key': '/etc/swftp/id_rsa.pub',
+    'num_persistent_connections': '100',
+    'num_connections_per_session': '10',
+    'connection_timeout': '240',
+    'verbose': 'false',
+
+    'log_statsd_host': '',
+    'log_statsd_port': '8125',
+    'log_statsd_sample_rate': '10.0',
+    'log_statsd_metric_prefix': 'swftp.sftp',
+
+    'stats_host': '',
+    'stats_port': '38022',
+}
+
+
 def run():
     options = Options()
     try:
@@ -31,21 +52,7 @@ def run():
 
 
 def get_config(config_path, overrides):
-    defaults = {
-        'auth_url': 'http://127.0.0.1:8080/auth/v1.0',
-        'host': '0.0.0.0',
-        'port': '5022',
-        'priv_key': '/etc/swftp/id_rsa',
-        'pub_key': '/etc/swftp/id_rsa.pub',
-        'num_persistent_connections': '4',
-        'connection_timeout': '240',
-        'verbose': 'false',
-        'log_statsd_host': '',
-        'log_statsd_port': '8125',
-        'log_statsd_sample_rate': '10.0',
-        'log_statsd_metric_prefix': 'swftp.sftp',
-    }
-    c = ConfigParser.ConfigParser(defaults)
+    c = ConfigParser.ConfigParser(CONFIG_DEFAULTS)
     c.add_section('sftp')
     if config_path:
         log.msg('Reading configuration from path: %s' % config_path)
@@ -83,29 +90,18 @@ class Options(usage.Options):
 def makeService(options):
     """
     Makes a new swftp-sftp service. The only option is the config file
-    location. The config file has the following options:
-     - host
-     - port
-     - auth_url
-     - num_persistent_connections
-     - connection_timeout
-     - pub_key
-     - priv_key
+    location. See CONFIG_DEFAULTS for list of configuration options.
     """
     from twisted.conch.ssh.factory import SSHFactory
     from twisted.conch.ssh.keys import Key
-    from twisted.web.client import HTTPConnectionPool
     from twisted.cred.portal import Portal
 
-    from swftp.sftp.server import SwiftSFTPRealm, SwiftSSHServerTransport
-    from swftp.sftp.connection import SwiftConnection
+    from swftp.sftp.server import SwiftSFTPRealm, SwiftSSHServerTransport, \
+        SwiftSSHConnection
     from swftp.auth import SwiftBasedAuthDB
-    from swftp.utils import print_runtime_info
+    from swftp.utils import log_runtime_info, GLOBAL_METRICS
 
     c = get_config(options['config_file'], options)
-    pool = HTTPConnectionPool(reactor, persistent=True)
-    pool.maxPersistentPerHost = c.getint('sftp', 'num_persistent_connections')
-    pool.cachedConnectionTimeout = c.getint('sftp', 'connection_timeout')
 
     sftp_service = service.MultiService()
 
@@ -113,7 +109,7 @@ def makeService(options):
     os.environ['TZ'] = 'GMT'
     time.tzset()
 
-    log.msg('Starting SwFTP-sftp %s' % VERSION)
+    print('Starting SwFTP-sftp %s' % VERSION)
 
     # Add statsd service
     if c.get('sftp', 'log_statsd_host'):
@@ -126,10 +122,34 @@ def makeService(options):
                 prefix=c.get('sftp', 'log_statsd_metric_prefix')
             ).setServiceParent(sftp_service)
         except ImportError:
-            log.err('Missing Statsd Module. Requires "txstatsd"')
+            sys.stderr.write('Missing Statsd Module. Requires "txstatsd" \n')
 
-    authdb = SwiftBasedAuthDB(auth_url=c.get('sftp', 'auth_url'),
-                              verbose=c.getboolean('sftp', 'verbose'))
+    if c.get('sftp', 'stats_host'):
+        from swftp.report import makeService as makeReportService
+        known_fields = [
+            'command.login',
+            'command.logout',
+            'command.gotVersion',
+            'command.openFile',
+            'command.removeFile',
+            'command.renameFile',
+            'command.makeDirectory',
+            'command.removeDirectory',
+            'command.openDirectory',
+            'command.getAttrs',
+        ] + GLOBAL_METRICS
+        makeReportService(
+            c.get('sftp', 'stats_host'),
+            c.getint('sftp', 'stats_port'),
+            known_fields=known_fields
+        ).setServiceParent(sftp_service)
+
+    authdb = SwiftBasedAuthDB(
+        c.get('sftp', 'auth_url'),
+        global_max_concurrency=c.getint('sftp', 'num_persistent_connections'),
+        max_concurrency=c.getint('sftp', 'num_connections_per_session'),
+        timeout=c.getint('sftp', 'connection_timeout'),
+        verbose=c.getboolean('sftp', 'verbose'))
 
     sftpportal = Portal(SwiftSFTPRealm())
     sftpportal.registerChecker(authdb)
@@ -138,7 +158,7 @@ def makeService(options):
     sshfactory.protocol = SwiftSSHServerTransport
     sshfactory.noisy = False
     sshfactory.portal = sftpportal
-    sshfactory.services['ssh-connection'] = SwiftConnection
+    sshfactory.services['ssh-connection'] = SwiftSSHConnection
 
     pub_key_string = file(c.get('sftp', 'pub_key')).read()
     priv_key_string = file(c.get('sftp', 'priv_key')).read()
@@ -147,8 +167,8 @@ def makeService(options):
     sshfactory.privateKeys = {
         'ssh-rsa': Key.fromString(data=priv_key_string)}
 
-    signal.signal(signal.SIGUSR1, print_runtime_info)
-    signal.signal(signal.SIGUSR2, print_runtime_info)
+    signal.signal(signal.SIGUSR1, log_runtime_info)
+    signal.signal(signal.SIGUSR2, log_runtime_info)
 
     internet.TCPServer(
         c.getint('sftp', 'port'),
