@@ -12,35 +12,14 @@ from twisted.web.client import Agent, WebClientContextFactory
 from twisted.internet.protocol import Protocol
 from twisted.web.http_headers import Headers
 from twisted.web import error
-from twisted.web.iweb import IBodyProducer
 from twisted.web._newclient import ResponseDone
 from twisted.web.http import PotentialDataLoss
 from twisted.python import log
 
-from zope.interface import implements
-
 import json
 from urllib import quote as _quote
 
 
-class StringProducer(object):
-    implements(IBodyProducer)
-
-    def __init__(self, body):
-        self.body = body
-        self.length = len(body)
-
-    def startProducing(self, consumer):
-        consumer.write(self.body)
-        return succeed(None)
-
-    def pauseProducing(self):
-        pass
-
-    def stopProducing(self):
-        pass
-
-
 class RequestError(error.Error):
     pass
 
@@ -74,7 +53,6 @@ class ResponseReceiver(Protocol):
         self.recv_chunks.append(bytes)
 
     def connectionLost(self, reason):
-
         if reason.check(ResponseDone) or reason.check(PotentialDataLoss):
             self.finished.callback(''.join(self.recv_chunks))
         else:
@@ -105,14 +83,14 @@ def cb_recv_resp(response, load_body=False, receiver=None):
     else:
         if receiver:
             response.deliverBody(receiver)
-            return
+            return response
         else:
             response.deliverBody(ResponseIgnorer(d_resp_recvd))
-    return d_resp_recvd.addCallback(cb_process_resp, response)
+    d_resp_recvd.addCallback(cb_process_resp, response)
+    return d_resp_recvd
 
 
 def cb_process_resp(body, response):
-    # Emulate HTTPClientFactory and raise t.w.e.Error if we have errors.
     if response.code == 404:
         raise NotFound(response.code, body)
     if response.code == 401:
@@ -143,8 +121,13 @@ def cb_json_decode(result):
 
 
 class SwiftConnection:
-    """
-        A basic connection class to interface with OpenStack Swift.
+    """ A basic connection class to interface with OpenStack Swift.
+
+        :param auth_url: auth endpoint for swift
+        :param username: username for swift
+        :param api_key: password/api_key for swift
+        :param pool: A twisted.web.client.HTTPConnectionPool object
+        :param bool verbose: verbose setting
     """
     user_agent = 'Twisted Swift'
 
@@ -154,36 +137,48 @@ class SwiftConnection:
         self.api_key = api_key
         self.storage_url = None
         self.auth_token = None
-        self.contextFactory = WebClientContextFactory()
-        self.contextFactory.noisy = False
+        contextFactory = WebClientContextFactory()
+        contextFactory.noisy = False
         self.pool = pool
+        self.agent = Agent(reactor, contextFactory, pool=self.pool)
         self.verbose = verbose
-        # self.agent = Agent(reactor, self.contextFactory, pool=pool)
 
-    def make_request(self, method, path, params=None, headers=None, body=None,
-                     body_reader=None):
-        h = {
-            'User-Agent': [self.user_agent],
-        }
-        if headers:
-            for k, v in headers.iteritems():
-                if not isinstance(v, list):
-                    h[k] = [v]
-                else:
-                    h[k] = v
+    def _form_url(self, path, params):
         url = "/".join((self.storage_url, path))
         if params:
             param_lst = []
             for k, v in params.iteritems():
                 param_lst.append("%s=%s" % (k, v))
             url = "%s?%s" % (url, "&".join(param_lst))
-        agent = Agent(reactor, self.contextFactory, pool=self.pool)
+        return url
+
+    def make_request(self, method, path, params=None, headers=None, body=None):
+        """ Make an HTTP request against Swift. This method will try once to
+        re-authenticate to swift after receiving a 401 or 403 and then
+        (if successful) will re-attempt the request.
+
+        :param method: HTTP Method. E.G. GET, POST, PUT
+        :param path: Path to be appended to the storage url
+        :param dict params: Parameters to be used at the query parameter
+        :param dict headers: Additional parameters for the request
+        :param body: Object which implements twisted.web.iweb.IBodyProducer
+
+        :returns t.w.c.Response:
+
+        """
+        h = {
+            'User-Agent': [self.user_agent],
+        }
+        if headers:
+            for k, v in headers.iteritems():
+                h[k] = [v]
 
         def doRequest(ignored):
             h['X-Auth-Token'] = [self.auth_token]
+            url = self._form_url(path, params)
             if self.verbose:
                 log.msg('Request: %s %s, headers: %s' % (method, url, h))
-            return agent.request(method, url, Headers(h), body)
+            return self.agent.request(method, url, Headers(h), body)
 
         d = doRequest(None)
 
@@ -209,68 +204,131 @@ class SwiftConnection:
         return result
 
     def authenticate(self):
+        """ Authenticate against Swift (using v1 auth)
+
+        :returns t.w.c.Response:
+
+        """
         headers = {
             'User-Agent': [self.user_agent],
             'X-Auth-User': [self.username],
             'X-Auth-Key': [self.api_key],
         }
-        agent = Agent(reactor, self.contextFactory, pool=self.pool)
-        d = agent.request('GET', self.auth_url, Headers(headers))
+        d = self.agent.request('GET', self.auth_url, Headers(headers))
         d.addCallback(cb_recv_resp, load_body=True)
         d.addCallback(self.after_authenticate)
         return d
 
     def head_account(self):
+        " Get details of the account "
         d = self.make_request('HEAD', '')
         d.addCallback(cb_recv_resp)
         d.addCallback(format_head_response)
         return d
 
-    def get_account(self, marker=None):
+    def get_account(self, limit=None, marker=None, end_marker=None):
+        """ Get listing of containers in the account
+
+        :param int limit: The max number of results to return
+        :param marker: container names greater than this value
+        :param end_marker: container names less than this value
+
+        :returns t.w.c.Response, list:
+
+        """
         params = {'format': 'json'}
+        if limit:
+            params['limit'] = str(limit)
         if marker:
             params['marker'] = quote(marker)
+        if marker:
+            params['end_marker'] = quote(end_marker)
+
         d = self.make_request('GET', '', params=params)
         d.addCallback(cb_recv_resp, load_body=True)
         d.addCallback(cb_json_decode)
         return d
 
     def head_container(self, container):
+        """ Get details on a container
+
+        :param container: The container name
+
+        :returns dict:
+
+        """
         d = self.make_request('HEAD', quote(container))
         d.addCallback(cb_recv_resp)
         d.addCallback(format_head_response)
         return d
 
-    def get_container(self, container, marker=None, prefix=None, path=None,
-                      delimiter=None, limit=None):
+    def get_container(self, container, limit=None, marker=None,
+                      end_marker=None, prefix=None, path=None, delimiter=None):
+        """ Create a container
+
+        :param container: The container name
+        :param int limit: The max number of results to return
+        :param marker: object names greater than this value
+        :param end_marker: object names less than this value
+        :param prefix: return objects with names that start with this value
+        :param delimiter: Delimiter to use for hierarchy
+
+        :returns t.w.c.Response, list:
+
+        """
         params = {'format': 'json'}
-        if marker:
-            params['marker'] = quote(marker)
-        if path:
-            params['path'] = quote(path)
-        if prefix:
-            params['prefix'] = quote(prefix)
-        if delimiter:
-            params['delimiter'] = quote(delimiter)
         if limit:
             params['limit'] = str(limit)
+        if marker:
+            params['marker'] = quote(marker)
+        if marker:
+            params['end_marker'] = quote(end_marker)
+        if prefix:
+            params['prefix'] = quote(prefix)
+        if path:
+            params['path'] = quote(path)
+        if delimiter:
+            params['delimiter'] = quote(delimiter)
+
         d = self.make_request('GET', quote(container), params=params)
         d.addCallback(cb_recv_resp, load_body=True)
         d.addCallback(cb_json_decode)
         return d
 
     def put_container(self, container, headers=None):
-        d = self.make_request('PUT', quote(container),
-                              headers=headers)
+        """ Create a container
+
+        :param container: The container name
+        :param header: Optional headers to add to the request
+
+        :returns t.w.c.Response:
+
+        """
+        d = self.make_request('PUT', quote(container), headers=headers)
         d.addCallback(cb_recv_resp)
         return d
 
     def delete_container(self, container):
+        """ Delete a container
+
+        :param container: The container name
+
+        :returns t.w.c.Response:
+
+        """
         d = self.make_request('DELETE', quote(container))
         d.addCallback(cb_recv_resp)
         return d
 
     def head_object(self, container, path):
+        """ Get details about an object
+
+        :param container: The container name
+        :param path: The object name/path
+
+        :returns dict:
+
+        """
         _path = "/".join((quote(container), quote(path)))
         d = self.make_request('HEAD', _path)
         d.addCallback(cb_recv_resp)
@@ -278,12 +336,33 @@ class SwiftConnection:
         return d
 
     def get_object(self, container, path, headers=None, receiver=None):
+        """ Download an object
+
+        :param container: The container name
+        :param path: The object name/path
+        :param dict headers: Extra headers to use with the HTTP request
+        :param receiver: A twisted.internet.protocol.Protocol that will receive
+                         the contents of the object
+
+        :returns t.w.c.Response:
+
+        """
         _path = "/".join((quote(container), quote(path)))
         d = self.make_request('GET', _path, headers=headers)
         d.addCallback(cb_recv_resp, receiver=receiver)
         return d
 
     def put_object(self, container, path, headers=None, body=None):
+        """ Create a new object
+
+        :param container: The container name
+        :param path: The object name/path
+        :param dict headers: Extra headers to use with the HTTP request
+        :param body: Object which implements twisted.web.iweb.IBodyProducer
+
+        :returns t.w.c.Response:
+
+        """
         if not headers:
             headers = {}
         if not body:
@@ -294,12 +373,58 @@ class SwiftConnection:
         return d
 
     def delete_object(self, container, path):
+        """ Delete an object
+
+        :param container: The container name
+        :param path: The object name/path
+
+        :returns t.w.c.Response:
+
+        """
         _path = "/".join((quote(container), quote(path)))
         d = self.make_request('DELETE', _path)
         d.addCallback(cb_recv_resp)
         return d
 
 
+class ThrottledSwiftConnection(SwiftConnection):
+    """ A SwiftConnection that has a list of locks that it needs to acquire
+        before making requests. Locks can either be a DeferredSemaphore, a
+        DeferredLock, or anything else that implements
+        twisted.internet.defer._ConcurrencyPrimitive. Locks are acquired in the
+        order in the list.
+
+        :param locks: list of locks that implement
+            twisted.internet.defer._ConcurrencyPrimitive
+        :param \*args: same arguments as `SwiftConnection`
+        :param \*\*args: same keyword arguments as `SwiftConnection`
+    """
+    def __init__(self, locks, *args, **kwargs):
+        SwiftConnection.__init__(self, *args, **kwargs)
+        self.locks = locks or []
+
+    def _release_all(self, result):
+        for i, lock in enumerate(self.locks):
+            lock.release()
+        return result
+
+    def _aquire_all(self):
+        d = succeed(None)
+        for i, lock in enumerate(self.locks):
+            d.addCallback(lambda r: lock.acquire())
+        return d
+
+    def make_request(self, *args, **kwargs):
+        def execute(ignored):
+            d = SwiftConnection.make_request(self, *args, **kwargs)
+            d.addBoth(self._release_all)
+            return d
+
+        d = self._aquire_all()
+        d.addCallback(execute)
+        return d
+
+
 def quote(value, safe='/'):
     """
     Patched version of urllib.quote that encodes utf8 strings before quoting
